/* isrLib.c
 *
 * This file contains routines to support registration of interrupt
 * handlers for any interrupt.
 */

#include "isrLib.h"

#if  defined(DEVPLATFORM331) || defined(DEVPLATFORM332) || defined(DEVPLATFORM336) || defined(DEVPLATFORM338) || defined(DEVPLATFORM376)
#define ANY_DEV_PLATFORM
#endif

#if defined(ANY_DEV_PLATFORM) || defined(CPU32SIM) || defined(TPUCPU32SIM) || defined(DBG3XX_SYSRAM)  || defined(TPUMASKACPU32SIM) || defined(TPUMASKGCPU32SIM) || defined(TPUMASK5XXSIM)
/* for this target, allow user full access to entire vector table */
#define FIRST_VEC_NUM	0
#define LAST_VEC_NUM	255
/* allocate the user vector table */
void* userVecTable[256];
#endif
#if defined(DBG3XX_TPURAM)
/* only support the first 16 user-defined interrupts */
#define FIRST_VEC_NUM	64
#define LAST_VEC_NUM	79
#define VECTOR_BASE_ADDR 0x70D6C0	/* this puts the first 16 user vecs at 0x70D7C0 - 0x70D800 */
void** userVecTable = (void**)VECTOR_BASE_ADDR;
#endif
#define TOTAL_VECS		(LAST_VEC_NUM - FIRST_VEC_NUM + 1)

/* interrupt vector/parameter tables */
ISR_FUNC_PTR isrAddrTable[TOTAL_VECS];
int isrParam1Table[TOTAL_VECS];
U32 isrParam2Table[TOTAL_VECS];

/* for assembly link-up - put table addresses into vars */
int isrAddrTableAddr;
int isrParam1TableAddr;
int isrParam2TableAddr;
int isrFirstVecNum;

/* user vector table support */
ISR_FUNC_PTR* vecTableBaseAddr = 0;

#ifndef WIN32
/* generic interrupt handler interface to C */

__asm__("

.text
.even
isrBase:

	/* save all registers (except SP) */
	/* could optimize this a bit if we know which registers are
	   guaranteed to be saved by a C-call */
	movem.l %d7/%d6/%d5/%d4/%d3/%d2/%d1/%d0/%a6/%a5/%a4/%a3/%a2/%a1/%a0,-(%a7)

	/* retrieve raw vector number data from exception stack frame, 
	   then compute vector number */
	move.l (0x40, %a7), %d0
	andi.l #0x3ff, %d0

	/* call C-code ISR */

	/* adjust index into table for first vec num in table */
	move.l isrFirstVecNum, %d2
	lsl.l #2, %d2
	sub.l %d2, %d0
	/* get ISR address from table*/
	move.l isrAddrTableAddr, %a0
	move.l (%a0, %d0), %a1
	/* get second handler parameter from table & push onto stack */
	move.l isrParam2TableAddr, %a0
	move.l (%a0, %d0), %d1
	move.l %d1, -(%a7)
	/* get first handler parameter from table & push onto stack */
	move.l isrParam1TableAddr, %a0
	move.l (%a0, %d0), %d1
	move.l %d1, -(%a7)
	/* call C-code ISR */
	jsr (%a1)
	/* restore stack ptr */
	/* move.l (%a7)+, %d0 */
    addq.l #8,%a7

	/* restore all registers */
	movem.l (%a7)+, %d7/%d6/%d5/%d4/%d3/%d2/%d1/%d0/%a6/%a5/%a4/%a3/%a2/%a1/%a0

	/* return from isr */
	rte

");
extern ISR_FUNC_PTR isrBase;
#else
ISR_FUNC_PTR isrBase;
#endif

/* this is the default ISR - does nothing (doesn't even clear interrupt) */
void defaultIsr(int isrParam)
{ 
	return;
}

/* Set the vector table base address */
void isrSetVecBase(ISR_FUNC_PTR* tableAddress)
{
	vecTableBaseAddr = tableAddress;
#ifndef WIN32
	/* set the VBR register */
	__asm__("

		move.l vecTableBaseAddr, %d0
		movec.l %d0, %vbr

	");
#endif
}

/* Initializes ISR support package.  All interrupts get assigned
 * the default handler. */
void isrLibInit()
{
	int i;

	isrSetVecBase((ISR_FUNC_PTR*)userVecTable);

	for (i = FIRST_VEC_NUM; i <= LAST_VEC_NUM; ++i)
	{
		isrAddrTable[i - FIRST_VEC_NUM] = defaultIsr;
		isrParam1Table[i - FIRST_VEC_NUM] = 0;
		isrParam2Table[i - FIRST_VEC_NUM] = 0;
		/* set the address in the actual table */
		*(vecTableBaseAddr + i) = (ISR_FUNC_PTR)&isrBase;
	}

	/* put table addresses into variables for asm link-up */
	isrAddrTableAddr = (int)isrAddrTable;
	isrParam1TableAddr = (int)isrParam1Table;
	isrParam2TableAddr = (int)isrParam2Table;
	isrFirstVecNum = FIRST_VEC_NUM;
}

/* connects a user isr routine to the specified interrupt vector */
void isrConnect(int vecNum, ISR_FUNC_PTR isrHandler, int isrPar1, U32 isrPar2)
{
	if (vecNum < FIRST_VEC_NUM || vecNum > LAST_VEC_NUM)
		return;

	if (isrHandler == 0)
		/* unregister any handler for vecNum */
		isrAddrTable[vecNum - FIRST_VEC_NUM] = defaultIsr;
	else
	{
		isrParam1Table[vecNum - FIRST_VEC_NUM] = isrPar1;
		isrParam2Table[vecNum - FIRST_VEC_NUM] = isrPar2;
		isrAddrTable[vecNum - FIRST_VEC_NUM] = isrHandler;
		*(vecTableBaseAddr + vecNum) = (ISR_FUNC_PTR)&isrBase;
	}
}

