/* main.c
 *
 * This application is written for the MPC5566EVB
 */

// for sim environment
#include "isrLib.h"
#include "ETpuLib.h"

// for eTPU/I2C
#include "etpu_util.h"
#include "etpu_init.h"
#include "etpu_i2c.h"
#include "etpu_i2c_master.h"
#include "etpu_i2c_slave.h"
#include "etpu_i2c_common.h"

uint8_t g_i2c_master_base_chan = 0;
uint8_t g_i2c_slave_base_chan = 4;

uint8_t g_i2c_error_flags;

uint8_t* g_p_i2c_master_cmd_buf;
uint8_t* g_p_i2c_master_buf1;
uint8_t* g_p_i2c_master_buf2;
uint8_t* g_p_i2c_slave_read_buf;
uint8_t* g_p_i2c_slave_write_buf;

uint32_t g_complete_flag = 0;


/* helper functions for interrupt polling - only for use on eTPU_A at this time */
uint32_t aw_etpu_i2c_master_wait_for_done_int(uint8_t channel)
{
	uint8_t error_flags;
	while (1)
	{
		uint32_t cisr;
		cisr = eTPU->CISR_A.R & (1 << (channel+ETPU_I2C_MASTER_SCL_OUT_OFFSET)); // look at master SCL_out interrupt bit
		if (cisr)
		{
			/* clear handled interrupts */
			eTPU->CISR_A.R = cisr;
			/* get error flags */
			if (aw_etpu_i2c_master_get_running_error_flags(channel, &error_flags))
				return 1;
			g_i2c_error_flags |= error_flags;
			break;
		}
	}
	return 0;
}
uint32_t aw_etpu_i2c_slave_wait_for_done_int(uint8_t channel)
{
	uint8_t error_flags;
	while (1)
	{
		uint32_t cisr;
		cisr = eTPU->CISR_A.R & (1 << (channel+ETPU_I2C_SLAVE_SDA_IN_OFFSET)); // look at slave SDA_in interrupt bit
		if (cisr)
		{
			/* clear handled interrupts */
			eTPU->CISR_A.R = cisr;
			/* get error flags */
			if (aw_etpu_i2c_master_get_running_error_flags(channel, &error_flags))
				return 1;
			g_i2c_error_flags |= error_flags;
			break;
		}
	}
	return 0;
}
uint32_t aw_etpu_i2c_slave_wait_for_data_request_int(uint8_t channel)
{
	while (1)
	{
		uint32_t cisr;
		cisr = eTPU->CISR_A.R & (1 << (channel+ETPU_I2C_SLAVE_SCL_IN_OFFSET)); // look at slave SCL_in interrupt bit
		if (cisr)
		{
			/* clear handled interrupts */
			eTPU->CISR_A.R = cisr;
			/* don't get errors at this time */
			break;
		}
	}
	return 0;
}


/* main application entry point */
int main()
{
	uint8_t header;
	uint32_t size;
	uint8_t i2c_local_buffer[64];
	uint8_t loop_cnt = 0;
	volatile U32 DelayCnt;

	/*
	 * Connections for master only test to Microchip PICkit Serial I2C Demo Board 
	 * ETPUA0 and ETPUA1 -> PIC 5
	 * ETPUA2 and ETPUA3 -> PIC 4

	 * Connections for master/slave loopback test
	 * ETPUA0 and ETPUA1 -> ETPUA4 and ETPUA5 w/ external pullup
	 * ETPUA2 and ETPUA3 -> ETPUA6 and ETPUA7 w/ external pullup
	 */

	/* Master port IOs */
	/* A0(SCL_OUT) */
	*(volatile U16 *)0xc3f90124 = 0x0623; // SIU_PCR114 register. Output open drain.
	/* A1(SCL_IN) */
	*(volatile U16 *)0xc3f90126 = 0x0503; // SIU_PCR115 register. Input.
	/* A2(SDA_OUT) */
	*(volatile U16 *)0xc3f90128 = 0x0623; // SIU_PCR116 register. Output open drain.
	/* A3(SDA_IN) */
	*(volatile U16 *)0xc3f9012a = 0x0503; // SIU_PCR117 register. Input.

	/* Slave port IOs */
	/* A4(SCL_IN) */
	*(volatile U16 *)0xc3f9012c = 0x0503; // SIU_PCR118 register. Output open drain.
	/* A5(SCL_OUT) */
	*(volatile U16 *)0xc3f9012e = 0x0623; // SIU_PCR119 register. Input.
	/* A6(SDA_IN) */
	*(volatile U16 *)0xc3f90130 = 0x0503; // SIU_PCR120 register. Output open drain.
	/* A7(SDA_OUT) */
	*(volatile U16 *)0xc3f90132 = 0x0623; // SIU_PCR121 register. Input.

	/* eTPU init */
	if (aw_etpu_init())
		return 1; // init failed

	/* Start the eTPU timers */
	fs_timer_start();

#if 1 
	/* Master only test to PIC board.
	 * This does two things. One part does a write/read test to the I2C EEPROM.
	 * The other part writes a binary sequence to the I2C port expander which can be seen on the LEDs. 
     */
	
	/* allocate the I2C data buffers first */
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_cmd_buf))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf1))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf2))
		return 1;

	/* init an I2C master at 100 KHz */
	if (aw_etpu_i2c_master_init(g_i2c_master_base_chan, 3, 100, g_p_i2c_master_cmd_buf))
		return 1;

	/* issue a transmit request to set the I2C port expander bits to outputs */
	g_p_i2c_master_buf1[0] = 0x00;
	g_p_i2c_master_buf1[1] = 0x00;
	if (aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x40, 2, g_p_i2c_master_buf1))
		return 1;
	
	/* Check for master transfer complete done */
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	if (g_i2c_error_flags != 0) while(1);
	
	while(1) {	
		/* Block write to EEPROM */
		g_p_i2c_master_buf1[0] = loop_cnt << 4;
		g_p_i2c_master_buf1[1] = (uint8_t)(0x11 + loop_cnt);
		g_p_i2c_master_buf1[2] = (uint8_t)(0x22 + loop_cnt);
		g_p_i2c_master_buf1[3] = (uint8_t)(0x33 + loop_cnt);
		g_p_i2c_master_buf1[4] = (uint8_t)(0x44 + loop_cnt);
		g_p_i2c_master_buf1[5] = (uint8_t)(0x55 + loop_cnt);
		g_p_i2c_master_buf1[6] = (uint8_t)(0x66 + loop_cnt);
		g_p_i2c_master_buf1[7] = (uint8_t)(0x77 + loop_cnt);
		g_p_i2c_master_buf1[8] = (uint8_t)(0x88 + loop_cnt);
		aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0xA0, 9, g_p_i2c_master_buf1);
		g_i2c_error_flags = 0;
		if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
			return 1;
		if (g_i2c_error_flags != 0) while(1);
		
		/* Wait for bytes to be written. The EEPROM will NAK it's address while it is busy. */
	 	int nak_cnt = -1; 
		do {
			nak_cnt++;
			g_p_i2c_master_buf1[0] = 0;
			aw_etpu_i2c_master_clear_running_error_flags(g_i2c_master_base_chan);
			aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0xA0, 0, g_p_i2c_master_buf1);
			g_i2c_error_flags = 0;
		 	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
				return 1;
		} while (g_i2c_error_flags);
	
		/* Read back the data */
	 	g_p_i2c_master_buf1[0] = loop_cnt << 4; 
	 	aw_etpu_i2c_master_combined_transfer(g_i2c_master_base_chan, 0xA0, 1, g_p_i2c_master_buf1, 0xA1, 8, g_p_i2c_master_buf2);
		g_i2c_error_flags = 0;
		if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
			return 1;
		if (g_i2c_error_flags != 0) while(1);
		
		/* Verify data */
		if ((g_p_i2c_master_buf2[0] != (uint8_t)(0x11 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[1] != (uint8_t)(0x22 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[2] != (uint8_t)(0x33 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[3] != (uint8_t)(0x44 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[4] != (uint8_t)(0x55 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[5] != (uint8_t)(0x66 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[6] != (uint8_t)(0x77 + loop_cnt)) ||
	   	   (g_p_i2c_master_buf2[7] != (uint8_t)(0x88 + loop_cnt))) {
	   	   while(1);
		}
		/* Write GPIOs */
		g_p_i2c_master_buf1[0] = 0x09;
		g_p_i2c_master_buf1[1] = loop_cnt;
		aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x40, 2, g_p_i2c_master_buf1);
		g_i2c_error_flags = 0;
		if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
			return 1;
		if (g_i2c_error_flags != 0) while(1);
		
		/* Read back GPIOs */
		g_p_i2c_master_buf1[0] = 0x09;
		aw_etpu_i2c_master_combined_transfer(g_i2c_master_base_chan, 0x40, 1, g_p_i2c_master_buf1, 0x41, 1, g_p_i2c_master_buf2);
		g_i2c_error_flags = 0;
		if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
			return 1;
		if (g_i2c_error_flags != 0) while(1);

		/* Verify GPIO data */
		if (g_p_i2c_master_buf2[0] != loop_cnt) {
		   while(1);
		}

		/* Delay time between loops */    
		for (DelayCnt = 0; DelayCnt < 0x200000; DelayCnt++);
		
		/* Increment for next loop */
		loop_cnt++;   
    }

#else /* Master slave loopback test */
	/* allocate the I2C data buffers first */
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_cmd_buf))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf1))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf2))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_slave_read_buf))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_slave_write_buf))
		return 1;

	/* init an I2C master at 100 KHz */
	if (aw_etpu_i2c_master_init(g_i2c_master_base_chan, 3, 100, g_p_i2c_master_cmd_buf))
		return 1;

	/* init I2C slave at 100 KHz */
	if (aw_etpu_i2c_slave_init(g_i2c_slave_base_chan, 3, 0x64, 0xfe, 1,
		ETPU_I2C_SLAVE_DATA_READY_FM0, g_p_i2c_slave_read_buf, 64,
		g_p_i2c_slave_write_buf, 64, 1250, 4700))
		return 1;

	/* issue a transmit request */
	g_p_i2c_master_buf1[0] = 0x11;
	g_p_i2c_master_buf1[1] = 0x22;
	g_p_i2c_master_buf1[2] = 0x33;
	g_p_i2c_master_buf1[3] = 0x44;
	if (aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x64, 4, g_p_i2c_master_buf1))
		return 1;

	/* first, check for master transfer complete done */
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	/* next check for slave transfer done interrupt */
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;
	/* check that received data is as expected, and no errors occurred */
	if ((g_p_i2c_slave_write_buf[0] != 0x11) || (g_p_i2c_slave_write_buf[1] != 0x22) || 
		(g_p_i2c_slave_write_buf[2] != 0x33) || (g_p_i2c_slave_write_buf[3] != 0x44) ||
		(g_i2c_error_flags != 0))
		return 1;
	aw_etpu_i2c_master_clear_running_error_flags(g_i2c_master_base_chan);
	aw_etpu_i2c_slave_clear_running_error_flags(g_i2c_slave_base_chan);

	/* check that retrieval of write buffer api works */
	if (aw_etpu_i2c_slave_get_write_data(g_i2c_slave_base_chan, &header, i2c_local_buffer, &size))
		return 1;
	if ((i2c_local_buffer[0] != 0x11) || (i2c_local_buffer[1] != 0x22) || 
		(i2c_local_buffer[2] != 0x33) || (i2c_local_buffer[3] != 0x44) ||
		(size != 4) || (header != 0x64))
		return 1;

	/* issue a read request */
	g_p_i2c_slave_read_buf[0] = 0x01;
	g_p_i2c_slave_read_buf[1] = 0x23;
	g_p_i2c_slave_read_buf[2] = 0x45;
	g_p_i2c_slave_read_buf[3] = 0x67;
	g_p_i2c_slave_read_buf[4] = 0x89;
	g_p_i2c_slave_read_buf[5] = 0xab;
	g_p_i2c_slave_read_buf[6] = 0xcd;
	g_p_i2c_slave_read_buf[7] = 0xef;
	if (aw_etpu_i2c_master_receive(g_i2c_master_base_chan, 0x65, 8, g_p_i2c_master_buf1))
		return 1;

	/* first, check for master transfer complete done */
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	/* next check for slave transfer done interrupt */
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;

	if ((g_p_i2c_master_buf1[0] != 0x01) || (g_p_i2c_master_buf1[1] != 0x23) || (g_p_i2c_master_buf1[2] != 0x45) || (g_p_i2c_master_buf1[3] != 0x67) ||
		(g_p_i2c_master_buf1[4] != 0x89) || (g_p_i2c_master_buf1[5] != 0xab) || (g_p_i2c_master_buf1[6] != 0xcd) || (g_p_i2c_master_buf1[7] != 0xef) ||
		(g_i2c_error_flags != 0))
		return 1;

	/* issue a combined transfer request */
	g_p_i2c_master_buf1[0] = 0x40;
	g_p_i2c_master_buf1[1] = 0x20;
	g_p_i2c_slave_read_buf[0] = 0xbe;
	g_p_i2c_slave_read_buf[1] = 0xef;
	if (aw_etpu_i2c_master_combined_transfer(g_i2c_master_base_chan, 0x64, 2, g_p_i2c_master_buf1,
		0x65, 2, g_p_i2c_master_buf2))
		return 1;

	/* first, check for master transfer complete done */
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	/* next check for slave transfer done interrupt */
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;

	/* verify that data was transferred correctly */
	if ((g_p_i2c_slave_write_buf[0] != 0x40) || (g_p_i2c_slave_write_buf[1] != 0x20))
		return 1;
	if ((g_p_i2c_master_buf2[0] != 0xbe) || (g_p_i2c_master_buf2[1] != 0xef))
		return 1;
	if (g_i2c_error_flags != 0)
		return 1;

	/* test wait for read */
	if (aw_etpu_i2c_shutdown(g_i2c_slave_base_chan))
		return 1;
	if (aw_etpu_i2c_slave_init(g_i2c_slave_base_chan, 3, 0x70, 0xf0, 1,
		ETPU_I2C_SLAVE_DATA_WAIT_FM0, g_p_i2c_slave_read_buf, 64,
		g_p_i2c_slave_write_buf, 64, 1250, 4700))
		return 1;	
	
	((aw_etpu_i2c_transfer_cmd*)g_p_i2c_master_cmd_buf)->_header = 0x75;
	((aw_etpu_i2c_transfer_cmd*)g_p_i2c_master_cmd_buf)->_p_buffer = (uint32_t)g_p_i2c_master_buf1 & 0x3fff;
	((aw_etpu_i2c_transfer_cmd*)g_p_i2c_master_cmd_buf)->_size = 3;
	if (aw_etpu_i2c_master_raw_transfer(g_i2c_master_base_chan, (aw_etpu_i2c_transfer_cmd*)g_p_i2c_master_cmd_buf, 1))
		return 1;

	/* wait for read interrupt, then supply read data and signal data ready */
	if (aw_etpu_i2c_slave_wait_for_data_request_int(g_i2c_slave_base_chan))
		return 1;
	/* write read data, and signal ready */
	g_p_i2c_slave_read_buf[0] = 0xaa;
	g_p_i2c_slave_read_buf[1] = 0x55;
	g_p_i2c_slave_read_buf[2] = 0xaa;
	aw_etpu_i2c_slave_issue_data_ready(g_i2c_slave_base_chan);

	/* check for master transfer complete done */
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;

	/* verify that data was transferred correctly */
	if ((g_p_i2c_master_buf1[0] != 0xaa) || (g_p_i2c_master_buf1[1] != 0x55) || (g_p_i2c_master_buf1[2] != 0xaa))
		return 1;
	if (g_i2c_error_flags != 0)
		return 1;

	/* test general call */
	g_p_i2c_master_buf1[0] = 0xcc;
	if (aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x00, 1, g_p_i2c_master_buf1))
		return 1;

	/* first, check for master transfer complete done */
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	/* next check for slave transfer done interrupt (both slaves) */
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;

	/* verify that slave received the general call */
	if (aw_etpu_i2c_slave_get_write_data(g_i2c_slave_base_chan, &header, i2c_local_buffer, &size))
		return 1;
	if ((i2c_local_buffer[0] != 0xcc) || (size != 1) || (header != 0x00))
		return 1;

	/* shutdown the I2C drivers */
	if (aw_etpu_i2c_shutdown(g_i2c_master_base_chan))
		return 1;
	if (aw_etpu_i2c_shutdown(g_i2c_slave_base_chan))
		return 1;

	/* TESTING DONE */

	g_complete_flag = 1;
	while (1);
#endif

	return 0;
}
