/**************************************************************************/
/* FILE NAME: mpc5500_xcptn.c               COPYRIGHT (c) Freescale 2004  */
/*                                                All Rights Reserved     */
/* DESCRIPTION:                                                           */
/* This file contains function calls for the MPC5500 exceptions and       */
/*  interrupts.                                                           */
/*========================================================================*/
/* REV      AUTHOR       DATE       DESCRIPTION OF CHANGE                 */
/* ---   -----------   ----------   ---------------------                 */
/* 0.1   G. Jackson    12/Jul/04     Initial version                      */ 
/* 1.0   G. Jackson    17/Aug/04     Added xcptn_xmpl function for        */
/*                                     exception and interrupt examples.  */
/* 1.1   C. Baker      19/Jul/06     Removed mpc5554.h #include, added    */
/*                                     software interrupt call            */
/**************************************************************************/

#include "mpc5500_usrccdcfg.h"
#include "mpc5500_xcptn.h"

/**************************************************************************/
/*                  C Code Variables and Definitions                      */
/**************************************************************************/
extern void IVOR4Handler (void);
extern vuint32_t __IV_ADDR;     /* Defined in the linker file */
extern uint32_t IntcIsrVectorTable[];

/* These two integer initializations are used with interrupts   */
/* Cannot initialize to zero until xcptn_xmpl( ) is called.*/
int emiosCh0Ctr;  /* Counter for eMIOS channel 0 interrupts */
int SWirq7Ctr;    /* Counter for software interrupt 7 */


/**************************************************************************/
/*                       C Code Functions                                 */
/**************************************************************************/

/************************************************************************/
/* FUNCTION     : initIrqVectors                                        */
/* PURPOSE      : This function intializes the IVPR and IVOR4 registers */
/*                  for exceptions.                                     */
/* INPUT NOTES  : None                                                  */
/* RETURN NOTES : None                                                  */
/* WARNING      : None                                                  */
/************************************************************************/

void initIrqVectors(void) {
  asm ("lis r5, __IV_ADDR@h");   /* IVPR = address base used with IVOR's */
  asm ("mtIVPR r5 ");
  asm ("lis r5, IVOR4Handler@h"); /* IVOR4 = address of handler */ 							
  asm ("ori r5, r5, IVOR4Handler@l");	
  asm ("mtIVOR4 r5");					
}

/**************************************************************************/
/* FUNCTION     : initINTC                                                */
/* PURPOSE      : This function intializes the INTC for software vector   */
/*                 mode.  Set up INTC vector table base address to start  */
/*                 at the beginning of L2SRAM.                            */
/* INPUT NOTES  : None                                                    */
/* RETURN NOTES : None                                                    */
/* WARNING      : None                                                    */
/**************************************************************************/
void initINTC(void) {	
  INTC.MCR.B.HVEN = 0;	/* Initialize INTC for software vector mode */
  INTC.MCR.B.VTES = 0;	/* Use the default vector table entry offsets of 4 bytes */
  INTC.IACKR.R = (uint32_t) &IntcIsrVectorTable[0]; /* Set INTC ISR vector table base addr. */
}

/**************************************************************************/
/* FUNCTION     : initEMIOS                                               */
/* PURPOSE      : This function intializes the EMIOS block for generating */
/*                 an interrupt.                                          */
/* INPUT NOTES  : None                                                    */
/* RETURN NOTES : None                                                    */
/* WARNING      : None                                                    */
/**************************************************************************/

void initEMIOS(void) {
  EMIOS.MCR.B.GPRE= 127;      /* Divide 128MHz sysclk by 127+1 for 1MHz eMIOS clk*/
  EMIOS.MCR.B.GPREN = 1;      /* Enable eMIOS clock */
  EMIOS.MCR.B.GTBE = 1;       /* Enable global time base */
  EMIOS.MCR.B.FRZ = 1;        /* Enable stopping channels when in debug mode */

                         /* Set up chan 0 for 1KHz output with IRQ */
  EMIOS.CH[0].CBDR.R = 1000;  /* Period is 1000 eMIOS clocks (1 msec) */
  EMIOS.CH[0].CADR.R = 500;   /* Duty cycle is 500 eMIOS clocks, 50% */
  EMIOS.CH[0].CCR.B.MODE = 0x18; /* Mode is Output pulse width + freq. modulation */
  EMIOS.CH[0].CCR.B.BSL = 0x3;   /* Use internal counter */
  EMIOS.CH[0].CCR.B.FREN = 1;	/* Stop (freeze) channel registers when in debug mode */
  EMIOS.CH[0].CCR.B.FEN=1;       /* Flag enables interrupt */
  EMIOS.CH[0].CCR.B.UCPREN = 1;  /* Enable prescaler; uses default divide by 1 */
  INTC.PSR[51].R = 1;            /* eMIOS chan 0 IRQ priority = 1 */
  SIU.PCR[191].R = 0x0303;       /* Setup GPIO OUTPUT on EMIOS12	*/
}


/**************************************************************************/
/* FUNCTION     : initSwIrq4                                              */
/* PURPOSE      : This function intializes interrupt 7 IRQ priority = 2   */
/*                                                                        */
/* INPUT NOTES  : None                                                    */
/* RETURN NOTES : None                                                    */
/* WARNING      : None                                                    */
/**************************************************************************/

void initSwIrq4(void) {
  INTC.PSR[4].R = 2;		/* software interrupt 7 IRQ priority = 2 */
}

/**************************************************************************/
/* FUNCTION     : emiosCh0ISR                                             */
/* PURPOSE      : This function increments the interrupt counter,         */
/*                 invokes software interrupt 4, clears the channel flag. */
/*                 eMIOS12 pin is inverted.                               */
/* INPUT NOTES  : None                                                    */
/* RETURN NOTES : None                                                    */
/* WARNING      : None                                                    */
/**************************************************************************/

void emiosCh0ISR(void) {
  emiosCh0Ctr++;			/* Increment interrupt counter */
  INTC.SSCIR[4].R = 2;  	/* Invoke software interrupt 4 */
  EMIOS.CH[0].CSR.B.FLAG=1;	/* Clear channel's flag */  
  SIU.GPDO[189].R = !SIU.GPDO[189].R;    /* Invert EMIOS10 (pin 189) */
}

/**************************************************************************/
/* FUNCTION     : SwIrq4ISR                                               */
/* PURPOSE      : This function increments the interrupt counter.         */
/*                                                                        */
/* INPUT NOTES  : None                                                    */
/* RETURN NOTES : None                                                    */
/* WARNING      : None                                                    */
/**************************************************************************/

void SwIrq4ISR(void) {
  SWirq7Ctr++;		 		/* Increment interrupt counter */
  INTC.SSCIR[4].R = 1;		/* Clear channel's flag */  
}

/************************************************************************/
/* FUNCTION     : enableIrq                                             */
/* PURPOSE      : This function sets INTC's current priority to 0.      */
/*                  External interrupts are enabled.                    */
/* INPUT NOTES  : None                                                  */
/* RETURN NOTES : None                                                  */
/* WARNING      : None                                                  */
/************************************************************************/

void enableIrq(void) {
  INTC.CPR.B.PRI = 0;	/* Ensure INTC's current priority is 0 */
  asm(" wrteei 1");		/* Enable external interrupts */
}

/******************************************************************************/
/* FUNCTION     : xcptn_xmpl                                                  */
/* PURPOSE      : This function serves as an example on setting up exceptions */
/*                  and interrupts in code.                                   */
/* INPUTS NOTES : None                                                        */ 
/* RETURN NOTES : None                                                        */
/* WARNING      : This function is called from main(). It is commented out to */
/*                  bypass the interrupt function.                            */
/******************************************************************************/

void xcptn_xmpl(void) {
    init_LED(PIN_EMIOS10, PIN_HI);   /* Initialize the GPIO ports for output. */
    initIrqVectors();         /* Initialize exceptions: load IVPR only for this example */
    initINTC();               /* Initialize INTC for software vector mode */
    initSwIrq4();			  /* Initialise Software Interrupt 4 for priority 2 */
    initEMIOS();              /* Initialize eMIOS channel 0 for 1KHz IRQ, priority 2 */
    enableIrq();              /* Ensure INTC current prority=0 & enable IRQ */

    emiosCh0Ctr = 0;          /* Counter for eMIOS channel 0 interrupts */
    SWirq7Ctr = 0;            /* Counter for software interrupt 7 */
}



/****************************************************/
/****************************************************/
