/* main_master_test.c
 *
 * This contains the entry point for the system -ash_main().  System init
 * is performed, then the main app is kicked off.
 *
 * This code performs various tests of the I2C master API.
 */

// for sim environment
#include "isrLib.h"
#include "ETpuLib.h"

// for eTPU/I2C
#include "etpu_util.h"
#include "etpu_init.h"
#include "etpu_i2c.h"
#include "etpu_i2c_master.h"
#include "etpu_i2c_slave.h"
#include "etpu_i2c_common.h"

/* set to 1 on succesful completion */
uint32_t g_complete_flag = 0;

uint8_t g_i2c_master_base_chan = 0;
uint8_t g_i2c_slave_base_chan = 10;

uint8_t g_i2c_error_flags;

uint8_t* g_p_i2c_master_cmd_buf;
uint8_t* g_p_i2c_master_buf1;
uint8_t* g_p_i2c_master_buf2;
uint8_t* g_p_i2c_master_buf3;
uint8_t* g_p_i2c_master_buf4;
uint8_t* g_p_i2c_slave_read_buf;
uint8_t* g_p_i2c_slave_write_buf;


/* helper functions for interrupt polling - only for use on eTPU_A at this time */
uint32_t aw_etpu_i2c_master_wait_for_done_int(uint8_t channel)
{
	uint8_t error_flags;
	while (1)
	{
		uint32_t cisr;
		cisr = eTPU->CISR_A.R & (1 << (channel+ETPU_I2C_MASTER_SCL_OUT_OFFSET)); // look at master SCL_out interrupt bit
		if (cisr)
		{
			/* clear handled interrupts */
			eTPU->CISR_A.R = cisr;
			/* get error flags */
			if (aw_etpu_i2c_master_get_running_error_flags(channel, &error_flags))
				return 1;
			g_i2c_error_flags |= error_flags;
			break;
		}
	}
	return 0;
}
uint32_t aw_etpu_i2c_slave_wait_for_done_int(uint8_t channel)
{
	uint8_t error_flags;
	while (1)
	{
		uint32_t cisr;
		cisr = eTPU->CISR_A.R & (1 << (channel+ETPU_I2C_SLAVE_SDA_IN_OFFSET)); // look at slave SDA_in interrupt bit
		if (cisr)
		{
			/* clear handled interrupts */
			eTPU->CISR_A.R = cisr;
			/* get error flags */
			if (aw_etpu_i2c_slave_get_running_error_flags(channel, &error_flags))
				return 1;
			g_i2c_error_flags |= error_flags;
			break;
		}
	}
	return 0;
}
uint32_t aw_etpu_i2c_slave_wait_for_data_request_int(uint8_t channel)
{
	while (1)
	{
		uint32_t cisr;
		cisr = eTPU->CISR_A.R & (1 << (channel+ETPU_I2C_SLAVE_SCL_IN_OFFSET)); // look at slave SCL_in interrupt bit
		if (cisr)
		{
			/* clear handled interrupts */
			eTPU->CISR_A.R = cisr;
			/* don't get errors at this time */
			break;
		}
	}
	return 0;
}


/* main application entry point */
/* w/ GNU, if we name this main, it requires linking with the libgcc.a
   run-time support.  This may be useful with C++ because this extra
   code initializes static C++ objects.  However, this C demo will
   skip it */
int ash_main()
{

	/* initialize interrupt support */
	isrLibInit();

	/* eTPU init */
	if (aw_etpu_init())
		return 1; // init failed

	/* start the eTPU timers */
	fs_timer_start();


	/* initialize I2C master (& slave for test purposes) */
	/* the eTPU script will connect the master and slave pins appropriately */
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_cmd_buf))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf1))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf2))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf3))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_master_buf4))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_slave_read_buf))
		return 1;
	if (aw_etpu_i2c_allocate_buffer(64, &g_p_i2c_slave_write_buf))
		return 1;

	/* test some bad params first */
	if (aw_etpu_i2c_master_init(30, 3, 400, g_p_i2c_master_cmd_buf) == 0) // should fail
		return 1;
	if (aw_etpu_i2c_master_init(g_i2c_master_base_chan, 5, 400, g_p_i2c_master_cmd_buf) == 0) // should fail
		return 1;
	if (aw_etpu_i2c_slave_init(29, 3, 0x64, 0xfe, 1,
		ETPU_I2C_SLAVE_DATA_READY_FM0, g_p_i2c_slave_read_buf, 64,
		g_p_i2c_slave_write_buf, 64, 400, 1300) == 0) // should fail
		return 1;
	if (aw_etpu_i2c_slave_init(g_i2c_slave_base_chan, 7, 0x64, 0xfe, 1,
		ETPU_I2C_SLAVE_DATA_READY_FM0, g_p_i2c_slave_read_buf, 64,
		g_p_i2c_slave_write_buf, 64, 400, 1300) == 0) // should fail
		return 1;

	if (aw_etpu_i2c_master_init(g_i2c_master_base_chan, 3, 400, g_p_i2c_master_cmd_buf))
		return 1;
	if (aw_etpu_i2c_slave_init(g_i2c_slave_base_chan, 3, 0x64, 0xfe, 1,
		ETPU_I2C_SLAVE_DATA_READY_FM0, g_p_i2c_slave_read_buf, 64,
		g_p_i2c_slave_write_buf, 64, 400, 1300))
		return 1;


	at_time(3000);

	/* test unexpected nacks & buffer overflow */

	/* issue a write request to slave addres that does not exist */
	g_p_i2c_master_buf1[0] = 0x11;
	g_p_i2c_master_buf1[1] = 0x22;
	g_p_i2c_master_buf1[2] = 0x33;
	g_p_i2c_master_buf1[3] = 0x44;
	if (aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x53, 4, g_p_i2c_master_buf1))
		return 1;
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	if (g_i2c_error_flags != ETPU_I2C_MASTER_ACK_FAILED)
		return 1;
	aw_etpu_i2c_master_clear_running_error_flags(g_i2c_master_base_chan);

	at_time(3500);

	/* issue a write request that overflows the slave's write buffer */
	if (aw_etpu_i2c_slave_set_write_buffer(g_i2c_slave_base_chan, g_p_i2c_slave_write_buf, 2))
		return 1;
	if (aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x64, 4, g_p_i2c_master_buf1))
		return 1;
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	// buffer overflow on slave does not result in untimely nack
	//if (g_i2c_error_flags != ETPU_I2C_MASTER_ACK_FAILED)
	if (g_i2c_error_flags != 0)
		return 1;
	aw_etpu_i2c_master_clear_running_error_flags(g_i2c_master_base_chan);
	g_i2c_error_flags = 0;
	// next check for slave transfer done interrupt
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;
	if (g_i2c_error_flags != ETPU_I2C_SLAVE_BUFFER_OVERFLOW)
		return 1;
	aw_etpu_i2c_slave_clear_running_error_flags(g_i2c_slave_base_chan);

	at_time(4000);

	/* test master busy */
	g_p_i2c_slave_read_buf[0] = 0x01;
	g_p_i2c_slave_read_buf[1] = 0x23;
	g_p_i2c_slave_read_buf[2] = 0x45;
	g_p_i2c_slave_read_buf[3] = 0x67;
	g_p_i2c_slave_read_buf[4] = 0x89;
	g_p_i2c_slave_read_buf[5] = 0xab;
	g_p_i2c_slave_read_buf[6] = 0xcd;
	g_p_i2c_slave_read_buf[7] = 0xef;
	if (aw_etpu_i2c_master_receive(g_i2c_master_base_chan, 0x65, 8, g_p_i2c_master_buf1))
		return 1;
	if (aw_etpu_i2c_master_transmit(g_i2c_master_base_chan, 0x53, 4, g_p_i2c_master_buf1) != FS_ETPU_ERROR_NOT_READY)
		return 1;
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	if ((g_p_i2c_master_buf1[0] != 0x01) || (g_p_i2c_master_buf1[1] != 0x23) || (g_p_i2c_master_buf1[2] != 0x45) || (g_p_i2c_master_buf1[3] != 0x67) ||
		(g_p_i2c_master_buf1[4] != 0x89) || (g_p_i2c_master_buf1[5] != 0xab) || (g_p_i2c_master_buf1[6] != 0xcd) || (g_p_i2c_master_buf1[7] != 0xef) ||
		(g_i2c_error_flags != 0))
		return 1;

	at_time(4500);

	/* test invalid START detection */
	wait_time(100);
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_slave_get_running_error_flags(g_i2c_slave_base_chan, &g_i2c_error_flags))
		return 1;
	if (g_i2c_error_flags != ETPU_I2C_SLAVE_INVALID_START)
		return 1;
	aw_etpu_i2c_slave_clear_running_error_flags(g_i2c_slave_base_chan);

	at_time(5000);

	/* test STOP failed detection */
	if (aw_etpu_i2c_master_receive(g_i2c_master_base_chan, 0x65, 1, g_p_i2c_master_buf1))
		return 1;
	if (aw_etpu_i2c_master_wait_for_done_int(g_i2c_master_base_chan))
		return 1;
	g_i2c_error_flags = 0;
	if (aw_etpu_i2c_slave_wait_for_done_int(g_i2c_slave_base_chan))
		return 1;
	if (g_i2c_error_flags != ETPU_I2C_SLAVE_STOP_FAILED)
		return 1;


	/* TESTING DONE */


	g_complete_flag = 1;
	while (1)
		;

	return 0;
}
