/**************************************************************************
* FILE NAME: etpu_i2c_master.h
* 
* DESCRIPTION: API for initializing and controlling the I2C eTPU function
* (I2C master driver)
*
*========================================================================
* REV      AUTHOR      DATE        DESCRIPTION OF CHANGE                 
* ---   -----------  ----------    ---------------------                 
* 1.0     J Diener   01/Dec/11     Initial version.     
*
* $Revision: 1.4 $
*
**************************************************************************/

#ifndef __ETPU_I2C_MASTER_H
#define __ETPU_I2C_MASTER_H

#include "typedefs.h"	/* type definitions for eTPU interface */

#ifdef __cplusplus
extern "C" {
#endif


// define the bitfield order for compiler
#define MSB_BITFIELD_ORDER
//#define LSB_BITFIELD_ORDER

// define the structure of a transfer command
typedef struct
{
#if defined(MSB_BITFIELD_ORDER)
	uint32_t _header : 8;
	uint32_t _p_buffer: 24;
#elif defined(LSB_BITFIELD_ORDER)
	uint32_t _p_buffer: 24;
	uint32_t _header : 8;
#else
#error Must define either MSB_BITFIELD_ORDER or LSB_BITFIELD_ORDER
#endif
	uint32_t _size;
} aw_etpu_i2c_transfer_cmd;


/****************************************************************
 * I2C Master eTPU app initialization.  This one routine initializes all
 * four eTPU channels that act together as an I2C master.  Four consecutive
 * channels must be used for an I2C master.
 *
 * channel - the base channel for the I2C Master eTPU instance.  The 
 * channels are defined as follows:
 *			channel   - SCL_out
 *			channel+1 - SCL_in
 *			channel+2 - SDA_out
 *			channel+3 - SDA_in
 * priority - the priority assigned to all the channels, with
 *		a range of 1 (low) to 3 (high).  The SCL_out channel is the primary
 *		that does most of the servicing and work, but they are
 *		assigned the same priority as SCL_out.
 * bit_rate_khz - the bit rate in kHz.  By default the initialization function
 *		derives all the various bit timings from this rate.  The
 *		aw_etpu_i2c_master_set_timing() interface can be used to override
 *		the default bit timing.
 * cmd_buffer_ptr - pointer to a buffer in eTPU data memory (SDM) that is to be 
 *		used for setting up transfer commands.  It should be at least 16 bytes
 *		in size in order to allow configuration of a combined transfer.
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_init(uint8_t channel,
								uint8_t priority,
								uint32_t bit_rate_khz,
								uint8_t* cmd_buffer_ptr);


/****************************************************************
 * Allows direct configuration of each timing parameter used in
 * the I2C master driver.
 *
 * channel - the I2C Master eTPU base channel of the instance
 * tLOW - the time in ns of low portion of the SCL clock signal.  tLOW + tHIGH
 *		equals the ideal bit time for the I2C master - the transfers will
 *		run at this rate unless a slave stretches the clock.
 * tHIGH - the time in ns of the high portion of the SCL clock signal.  tLOW + tHIGH
 *		equals the ideal bit time for the I2C master - the transfers will
 *		run at this rate unless a slave stretches the clock.
 * tBUF - bus free time between a STOP and START condition, in ns.
 * tSU_STA - set-up time for a repeated START, in ns.
 * tSU_STO - set-up time for a STOP, in ns.
 * tHD_DAT - data hold time (generally can be quite close to 0) in ns.
 * tr_max - maximum rise time for the signals, in ns.
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_set_timing(uint8_t channel,
									  uint32_t tLOW,
									  uint32_t tHIGH,
									  uint32_t tBUF,
									  uint32_t tSU_STA,
									  uint32_t tSU_STO,
									  uint32_t tHD_DAT,
									  uint32_t tr_max);

/****************************************************************
 * Transmit a buffer of data to the specified slave address.  When
 * transmission is complete, a channel interrupt will be generated
 * from the base channel.
 *
 * channel - the I2C Master eTPU base channel of the instance
 * slave_address - the slave address, where it is assumed bit 0
 *		is 0 and will be filled in based upon read/write.
 * buf_size - the size in bytes of the data to be transmitted.  It
 *		does not include the header byte that is made out of the
 *		slave address.
 * buffer_ptr - the buffer from which data is transmitted.  The buffer
 *		must reside in eTPU data memory (SDM).
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_transmit(uint8_t channel,
									uint8_t slave_address,
									uint32_t buffer_size,
									uint8_t* buffer_ptr
									);


/****************************************************************
 * Receive a data message from the specified slave address.  When
 * receipt is complete, a channel interrupt will be generated
 * from the base channel.
 *
 * channel - the I2C Master eTPU base channel
 * slave_address - the slave address, where it is assumed bit 0
 *		is 0 and will be filled in based upon read/write.
 * buf_size - the size in bytes of the data to be received.  It
 *		does not include the header byte that is made out of the
 *		slave address.
 * buffer_ptr - the buffer into which data is to be received.  The buffer
 *		must reside in eTPU data memory (SDM).
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_receive(uint8_t channel,
								   uint8_t slave_address,
								   uint32_t buffer_size,
								   uint8_t* buffer_ptr
								   );


/****************************************************************
 * Perform a combined transfer using the specified slave address.
 * The combined transfer allows for 2 back-to-back data transfers,
 * each of which can be a read or write transaction, independently.
 * When the combined transfer completes, a channel interrupt is
 * generated from the base channel of the I2C instance.
 *
 * channel - the I2C Master eTPU base channel
 * header1 - the header (address and R/W bit) for the first transfer.
 * buf1_size - the size in bytes of the data to be received or 
 *		transmitted in the first half of the combined transfer. It
 *		does not include the header byte that is made out of the
 *		slave address.
 * buf1_ptr - the buffer for data receipt/transmit for the first
 *		transfer.  Buffer must be in eTPU data memory (SDM).
 * header2 - the header (address and R/W bit) for the second transfer.
 * buf2_size - the size in bytes of the data to be received or 
 *		transmitted in the second half of the combined transfer. It
 *		does not include the header byte that is made out of the
 *		slave address.
 * buf2_ptr - the buffer for data receipt/transmit for the second
 *		transfer.  Buffer must be in eTPU data memory (SDM).
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_combined_transfer(uint8_t channel,
											 uint8_t header1,
											 uint32_t buf1_size,
											 uint8_t* buf1_ptr,
											 uint8_t header2,
											 uint32_t buf2_size,
											 uint8_t* buf2_ptr
											 );

/****************************************************************
 * Raw interface for setting up any kind of combined transfer.
 * The driver will process each of the cmd_cnt transfer commands
 * in sequence with a repeated START separating them.  When the full
 * transfer completes, a channel interrupt is generated from the base
 * channel of the I2C instance.
 *
 * channel - the I2C Master eTPU base channel
 * cmd_buffer_ptr - a buffer in eTPU data memory (SDM) that holds 
 *		cmd_cnt transfer commands (see the type aw_etpu_i2c_transfer_cmd
 *		for the layout of each command.
 * cmd_cnt - number of commands in the cmd_buffer.
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_raw_transfer(uint8_t channel,
										aw_etpu_i2c_transfer_cmd* cmd_buffer_ptr,
										uint32_t cmd_cnt);


/****************************************************************
 * Latch, clear and get the error flags associated with an I2C transfer.
 * The "latch and clear" interface does coherently latch the error
 * flags and clear the running error flags.  Otherwise the get and clear
 * are not coherent, but they generally don't need to be since
 * the user should only be calling this before or after a transfer,
 * not during.  Error flag meanings can be found in etpu_i2c_common.h.
 *
 * channel - the I2C Master eTPU base channel
 * error_flags_ptr - the byte location at which to write the error flags
 *		retrieved from the eTPU.  If a NULL pointer is passed, only
 *		the error flag clear is done.
 *
 * Returns failure code, or pass (0).
 ****************************************************************/
int32_t aw_etpu_i2c_master_latch_clear_error_flags(uint8_t channel);
int32_t aw_etpu_i2c_master_get_running_error_flags(uint8_t channel,
												   uint8_t* error_flags_ptr);
int32_t aw_etpu_i2c_master_clear_running_error_flags(uint8_t channel);
int32_t aw_etpu_i2c_master_get_latched_error_flags(uint8_t channel,
												   uint8_t* error_flags_ptr);
int32_t aw_etpu_i2c_master_clear_latched_error_flags(uint8_t channel);


#ifdef __cplusplus
}
#endif

#endif // __ETPU_I2C_MASTER_H
